var ZhFeatures = function(ApplicationContext) {
	var _usePostData = false;
	var socketService;
	var userId = localStorage.getItem("USER_ID");

	function getTemplateManager() {
		return ApplicationContext.templateManager;
	}

	this.__defineGetter__("usePostData", function() {
		if (_usePostData) {
			var message = "The feature [usePostData] will be removed on 2014-12-21 please migrate to new backEnd!";
			console.warn(message);
		}
		return _usePostData;
	});

	this.__defineSetter__("usePostData", function(val) {
		_usePostData = val;
	});
	this.useMappedPackagesUrlWithStringUrl = function(){
		var message = "The feature [useMappedPackagesUrlWithStringUrl] will be removed on 2016-04-21 please take a lock at your !";
		console.warn(message);
	};


	/*
	 * Registers information about a feature usage
	 *
	 * @param additionalContext additional information about the context in which the function was called
	 * @example zhFeatures.registerFeatureUsage({
	 * 	directive: "myDirective",
	 * 	action: "myAwesomeAction",
	 * 	userLogin: "abc123"
	 * });
	 */
	this.registerFeatureUsage = function registerFeatureUsage(additionalContext) {
		var templateManager = getTemplateManager();
		if (templateManager.project.zhFeatureUsageEnabled) {
			var featureContext = getBaseFeatureContext(getCallerNameOrContent(arguments.callee.caller));
			if (additionalContext) {
				addAdditionalFeatureContext(featureContext, additionalContext);
			}

			if (!socketService && ApplicationContext.ZhSocketFactory) {
				socketService = new ZhFeaturesSocketService(ApplicationContext);
			}

			if (socketService) {
				socketService.connectionPromise.then(function () {
					socketService.sendMessage({data: featureContext});
				});
			}
		}
	};

	function getCallerNameOrContent(caller) {
		return caller? caller.name || caller.toString().replace(/ /g, '') : '';
	}

	function getBaseFeatureContext(caller) {
		var templateManager = getTemplateManager();
		var featureContext = {
			userId: userId,
			projectName: templateManager.project.name,
			containerName: templateManager.container.name,
			versionData: templateManager.versionData,
			caller: caller,
			timestamp: new Date().getTime()
		};

		return featureContext;
	}

	function addAdditionalFeatureContext(featureContext, additionalContext) {
		Object.keys(additionalContext).forEach(function (contextKey) {
			featureContext[contextKey] = additionalContext[contextKey];
		});
	}

	/*
	 * The Socket Service to be used only inside ZhFeatures for features gathering
	 */
	function ZhFeaturesSocketService(ApplicationContext) {
		var self = this;
		var zhPromise = ApplicationContext.ZHPromise;
		var templateManager = ApplicationContext.templateManager;
		var zhSocketFactory = ApplicationContext.ZhSocketFactory;
		var connectionDeferred = zhPromise.defer();
		this.connectionPromise = connectionDeferred.promise;
		var webSocket = null;

		this.sendMessage = function (message) {
			this.connectionPromise.then(function () {
				message.data = message.data || {};
				message.namespace = "zh_features";
				message.room =  message.room || templateManager.project.zhFeaturesWebSocketRoom || "zeedhi_features_usage";
				message.data.key = "you can create!";
				if (!message.event) {
					message.event = message.data.event = "registerFeatureUsage";
				}

				webSocket.send(stringifyAndRemoveInvalidCharacters(message));
			});
		};

		function stringifyAndRemoveInvalidCharacters(message) {
			return JSON.stringify(message).replace(/\$/g, "_");
		}

		this.prepareSocketConnection = function() {
			var parepareSocketEvents = ['createNamespace', 'createRoom'];

			parepareSocketEvents.forEach(function (eventName) {
				var message = {
					data: {}
				};
				message.data.event = message.event = eventName;
				self.sendMessage(message);
			});
		};

		this.connect = function(client) {
			var webSocketUrl = templateManager.project.zhFeaturesWebSocketUrl;
			if (client) {
					webSocket = zhSocketFactory.create(client, webSocketUrl);
					webSocket.addListener({ onOpen: onOpen }, {connectionLost: onConnectionLost});
			}
		};

		function onOpen() {
			connectionDeferred.resolve();
			self.prepareSocketConnection();
		}

		function onConnectionLost() {
			connectionDeferred = zhPromise.defer();
			this.connectionPromise = connectionDeferred.promise;
		}

		this.connect(ZhFeaturesResponseHandler);
	}

	function ZhFeaturesResponseHandler() {
		// For now we will just send information to the websocket
	}
};